﻿/* Downloaded from http://karmian.org */
using System;
using System.Collections.Generic;
using System.Linq;
using Karmian.Framework.Helpers;

namespace KarmianEqualityComparer
{
	internal class Program
	{
		private static void Main()
		{
			var cities = CreateCities();
			Console.WriteLine("Normal listing:");
			foreach (var city in cities)
				Console.WriteLine(city);
			Console.WriteLine("Press any key to continue...\n");
			Console.ReadKey(true);

			Console.WriteLine("Distinct by name:");
			foreach (var city in cities.Distinct(
				new LambdaComparer<City>(
					(c1, c2) => c1.Name.Equals(c2.Name),
					c => 1 /* force compare other than by instance */)))
				Console.WriteLine(city);
			Console.WriteLine("Press any key to continue...\n");
			Console.ReadKey(true);

			Console.WriteLine("More than one property is duplicated:");
			foreach (var city in cities.Distinct(
				new LambdaComparer<City>(
					(c1, c2) => c1.Population.Equals(c2.Population) || c2.Name.Equals(c2.Name),
					c => 1 /* force compare other than by instance */)))
				Console.WriteLine(city);
			Console.WriteLine("Press any key to continue...\n");
			Console.ReadKey(true);

			Console.WriteLine("Contains city with population of 789? :");
			Console.WriteLine(cities.Contains(
				new City { Name = "City789", Population = 789 },
				new LambdaComparer<City>(
					(c1, c2) => c1.Population.Equals(c2.Population),
					c => c.GetHashCode() /* instance has nothing to do here */)));
			Console.WriteLine("Press any key to continue...\n");
			Console.ReadKey(true);

			Console.WriteLine("HashSet with equality set to City.Name property? :");
			var hashSet = new HashSet<City>(
				new LambdaComparer<City>(
					(c1, c2) => c1.Name.Equals(c2.Name),
					c => 1 /* force compare other than by instance */))
			        {
			            new City {Name = "aaa"},
			            new City {Name = "aba"},
			            new City {Name = "aaa"},
			            new City {Name = "baa"},
			            new City {Name = "aba"}
			        };
			foreach (var city in hashSet)
				Console.WriteLine(city);
			Console.WriteLine("Press any key to exit...\n");
			Console.ReadKey(true);
		}

		private static IEnumerable<City> CreateCities()
		{
			return new List<City>
			{
			    new City { Name = "City1", Population = 123 },
			    new City { Name = "City2", Population = 456 },
			    new City { Name = "City1", Population = 789 },
			    new City { Name = "City3", Population = 123 },
			    new City { Name = "City4", Population = 987 },
			    new City { Name = "City4", Population = 654 }
			};
		}
	}
}